using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Diagnostics; 
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinSchedule;
using Infragistics.Win.UltraWinSchedule.TaskUI;
using Infragistics.Win.UltraWinEditors; 
using Infragistics.Win.UltraWinGanttView;
using Infragistics.Win.UltraWinTabControl; 
using GanttChart.Model;
using GanttChart.View;

namespace GanttChart.ViewModel
{
    public partial class BindingGanttView : Form
    {
 
        #region Constructor

        public BindingGanttView()
        {
            InitializeComponent();
        }


        #endregion

        #region Private Methods


        #endregion

        #region Form Load Event

        private void BindingGanttView_Load(object sender, EventArgs e)
        {

            //  Set the BindingContextControl property to reference this form
            this.ultraCalendarInfo1.DataBindingsForTasks.BindingContextControl = this;
            this.ultraCalendarInfo1.DataBindingsForProjects.BindingContextControl = this;

            // Set the DataSource and DataMember for Project and Tasks using the SetDataBinding method
            this.ultraCalendarInfo1.DataBindingsForProjects.SetDataBinding(ProjectRepository.Instance.battlePlanRepo, "BattlePlans");
            this.ultraCalendarInfo1.DataBindingsForTasks.SetDataBinding(ProjectRepository.Instance.battlePlanRepo, "Tasks");

            //  Set the DataBinding members for Projects 
            this.ultraCalendarInfo1.DataBindingsForProjects.IdMember = "PlanGuid";
            this.ultraCalendarInfo1.DataBindingsForProjects.KeyMember = "PlanID";
            this.ultraCalendarInfo1.DataBindingsForProjects.NameMember = "PlanName";
            this.ultraCalendarInfo1.DataBindingsForProjects.StartDateMember = "PlanStartTime";

            //  Set the DataBinding members for Tasks
            // Basic Task properties 
            this.ultraCalendarInfo1.DataBindingsForTasks.NameMember = "TaskName"; 
            this.ultraCalendarInfo1.DataBindingsForTasks.DurationMember = "TaskDuration"; 
            this.ultraCalendarInfo1.DataBindingsForTasks.StartDateTimeMember = "TaskStartTime"; 
            this.ultraCalendarInfo1.DataBindingsForTasks.IdMember = "TaskGuid"; 
            this.ultraCalendarInfo1.DataBindingsForTasks.ProjectKeyMember = "PlanID"; 
            this.ultraCalendarInfo1.DataBindingsForTasks.ParentTaskIdMember = "ParentTaskGuid";  
            this.ultraCalendarInfo1.DataBindingsForTasks.PercentCompleteMember = "TaskPercentComplete";
            this.ultraCalendarInfo1.DataBindingsForTasks.AllPropertiesMember = "AllProperties"; 
                 
            // All other properties
            this.ultraCalendarInfo1.DataBindingsForTasks.AllPropertiesMember = "AllProperties";

            //  Since we are showing a task that belongs to an explicitly defined
            //  project (i.e., not the UnassignedProject), assign that project to
            //  the control's Project property so the control knows to display that project.
            this.ultraGanttView1.CalendarInfo = this.ultraCalendarInfo1;
            Project project = new Project(); 
            project.Key = ProjectRepository.Instance.defaultProjectName; 
            project.Name = ProjectRepository.Instance.defaultProjectName; 
            project.StartDate = DateTime.Now; 
            this.ultraCalendarInfo1.Projects.Add(project);
            this.ultraGanttView1.Project = this.ultraGanttView1.CalendarInfo.Projects[1]; 
            
            this.ultraGanttView1.CreationFilter = new CurrentDayIndicatorCreationFilter();
            this.ultraGanttView1.GridAreaWidth = 400;

            ultraGrid1.DataSource = ProjectRepository.Instance.projectPlanRepo;
            ultraGrid1.DataMember = "Tasks"; 
            
        }

        #endregion

        #region //Event Handlers
        private void ultraGanttView1_MessageReceived(object sender, EventArgs e)
        {
        }

        private void ultraGanttView1_Click(object sender, EventArgs e)
        {
        }

        public void ultraGanttView1_PropertyChanged(object sender, Infragistics.Win.PropertyChangedEventArgs e)
        {
        }

        public void ultraGanttView1_ContextMenuInitializing(object sender, Infragistics.Win.UltraWinGanttView.ContextMenuInitializingEventArgs e)
        {
        }
        
        public void ultraGanttView1_TaskDialogDisplaying(object sender, Infragistics.Win.UltraWinGanttView.TaskDialogDisplayingEventArgs e)
        {
        }

        public void ultraGanttView1_TaskDataAdded(object sender, TaskAddedEventArgs e)
        {
        }

        public void ultraGanttView1_TaskAdded(object sender, TaskAddedEventArgs e)
        {
            e.Task.SetDuration(e.Task.Duration, TimeSpanFormat.Minutes);
        }

        private void ultraCalendarInfo1_TaskDataInitialized(object sender, TaskDataInitializedEventArgs e)
        {

            try
            {
                e.Task.SetDuration(e.Task.Duration, TimeSpanFormat.Minutes);
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Error changing task duration from days to minutes");
                Debug.WriteLine(ex.GetType().Name);
                Debug.WriteLine(ex.Message);
            }
        }

        #endregion //Event Handlers
    }
}
