using System;
using System.Threading; 
using System.Collections.Generic;
using System.Text;
using System.Data;
using GanttChart.Model; 
using GanttChart.DataAccess.COMInterface; 

namespace GanttChart.DataAccess
{
    public class COM_Component
    {

        private Guid _mostRecentlyParsedProjectGuid;
        public Guid mostRecentlyParsedProjectGuid
        {
            get { return _mostRecentlyParsedProjectGuid; }
            set { _mostRecentlyParsedProjectGuid = value; }
        }

        private Guid _mostRecentlyParsedPhaseGuid;
        public Guid mostRecentlyParsedPhaseGuid
        {
            get { return _mostRecentlyParsedPhaseGuid; }
            set { _mostRecentlyParsedPhaseGuid = value;}
        }

        private DateTime _mostRecentlyParsedPhaseStartTime;
        public DateTime mostRecentlyParsedPhaseStartTime
        {
            get { return _mostRecentlyParsedPhaseStartTime; }
            set { _mostRecentlyParsedPhaseStartTime = value; }
        }


        // Subscribers
        public GanttChart.DataAccess.COMInterface.CreateProjectRequestTypeSubscriber createProjectRequestTypeSubscriber;
        public GanttChart.DataAccess.COMInterface.DefinePhaseRequestTypeSubscriber definePhaseRequestTypeSubscriber;
        public GanttChart.DataAccess.COMInterface.DefineTaskRequestTypeSubscriber defineTaskRequestTypeSubscriber;

        public COM_Component()
        {
        }

        public void StartListen()
        {
            //CreateProjectRequestType
            createProjectRequestTypeSubscriber = new CreateProjectRequestTypeSubscriber(this);
            //DefinePhaseRequestType
            definePhaseRequestTypeSubscriber = new DefinePhaseRequestTypeSubscriber(this);
            //DefineTaskRequestType
            defineTaskRequestTypeSubscriber = new DefineTaskRequestTypeSubscriber(this);
        }

        public void OnDataReceived(UCS.CreateProjectRequestType msg)
        {
            // Push data to Gantt Chart
            var selectStatement = string.Format("TaskName = '{0}'", new string(msg.projectPlanUniqueID.Value));
            DataRow[] rows = ProjectRepository.Instance.theTasks.Select(selectStatement);

            // insert new if taskName is not in the table
            if (rows.Length < 1)
            {
                DataRow newRow = ProjectRepository.Instance.theTasks.NewRow();
                Guid thisGuid = Guid.NewGuid();
                newRow["TaskGuid"] = thisGuid;
                mostRecentlyParsedProjectGuid = thisGuid;
                Console.WriteLine("Writing project with guid: " + thisGuid);
                newRow["PlanID"] = ProjectRepository.Instance.defaultProjectName;
                newRow["TaskName"] = new string(msg.projectPlanUniqueID.Value);
                newRow["ParentTaskGuid"] = null;                        // Project tasks never have parents in the Gantt Chart
                Console.WriteLine("Aligned under null parent guid");
                newRow["TaskStartTime"] = DateTime.UtcNow;
                newRow["TaskDuration"] = TimeSpan.FromMinutes(1);
                newRow["TaskPercentComplete"] = 0;
                newRow["TaskType"] = GanttChart.Model.TaskType.Project; 
                ProjectRepository.Instance.theTasks.Rows.Add(newRow);
            }
        }

        public void OnDataReceived(UCS.DefinePhaseRequestType msg)
        { 
            // Push data to Gantt Chart
            var selectStatement = string.Format("TaskName = '{0}'", new string(msg.phase.phaseName));
            DataRow[] rows = ProjectRepository.Instance.theTasks.Select(selectStatement);

            // insert new if taskName is not in the table
            if (rows.Length < 1)
            {
                DataRow newRow = ProjectRepository.Instance.theTasks.NewRow();
                Guid thisGuid = Guid.NewGuid();
                newRow["TaskGuid"] = thisGuid;
                mostRecentlyParsedPhaseGuid = thisGuid;
                newRow["PlanID"] = ProjectRepository.Instance.defaultProjectName;
                newRow["TaskName"] = new string(msg.phase.phaseName);
                newRow["ParentTaskGuid"] = mostRecentlyParsedProjectGuid;
                Console.WriteLine("|--->Parent guid: " + mostRecentlyParsedProjectGuid + ", this guid: " + thisGuid);
                mostRecentlyParsedPhaseStartTime = ConvertToDateTime(msg.phaseParameters.phaseStartTime); 
                DateTime finishTime = ConvertToDateTime(msg.phaseParameters.phaseFinishTime);
                TimeSpan duration = finishTime - mostRecentlyParsedPhaseStartTime;
                newRow["TaskStartTime"] = mostRecentlyParsedPhaseStartTime; 
                newRow["TaskDuration"] = duration;
                newRow["TaskPercentComplete"] = 0;
                newRow["TaskType"] = GanttChart.Model.TaskType.Phase; 
                ProjectRepository.Instance.theTasks.Rows.Add(newRow);
            }
        }

        public void OnDataReceived(UCS.DefineTaskRequestType msg)
        {
            // Push data to Gantt Chart
            var selectStatement = string.Format("TaskName = '{0}'", new string(msg.task.taskName));
            DataRow[] rows = ProjectRepository.Instance.theTasks.Select(selectStatement);

            // insert new if taskName is not in the table
            if (rows.Length < 1)
            {
                DataRow newRow = ProjectRepository.Instance.theTasks.NewRow();
                Guid thisGuid = Guid.NewGuid(); 
                newRow["TaskGuid"] = thisGuid;
                newRow["PlanID"] = ProjectRepository.Instance.defaultProjectName;
                newRow["TaskName"] = new string(msg.task.taskName);
                newRow["ParentTaskGuid"] = mostRecentlyParsedPhaseGuid;  // DefineTaskRequestTypes always have phases as parents in the Gantt Chart
                Console.WriteLine("|------>Parent guid: " + mostRecentlyParsedPhaseGuid + " this guid: " + thisGuid);
                DateTime specificStartTime = mostRecentlyParsedPhaseStartTime.AddSeconds(msg.taskParameters.startDelay);
                newRow["TaskStartTime"] = specificStartTime;
                newRow["TaskDuration"] = TimeSpan.FromDays(msg.taskParameters.dwellTime/86400);  //convert seconds to days
                newRow["TaskPercentComplete"] = 0;
                newRow["TaskType"] = GanttChart.Model.TaskType.BaseTask; 
                ProjectRepository.Instance.theTasks.Rows.Add(newRow);
            }
        }

        DateTime ConvertToDateTime(double unixTime)
        {
            var epoch = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);
            return epoch.AddSeconds(unixTime);
        }
    }
}
