
using System;
using System.Data;
using System.Collections.Generic;
using GanttChart.DataAccess;
using GanttChart.DataAccess.DDSInterface;
using UCS.AMT;

namespace GanttChart.Model
{
    enum TaskType { Project, Phase, BaseTask }; 

    public class ProjectRepository
    {
        //All
        public COM_Component component;
        public String defaultProjectName = "db";
        public DataSet projectPlanRepo;
        public DataTable thePlans;
        public DataTable theTasks;

        private static ProjectRepository _instance = null;
        private static readonly object _locker = new object();

        private ProjectRepository()
        {
            InitializeRepos();
            lock (_locker)
            {
                component = new COM_Component();
                //component.StartListen();
            }
        }

        public static ProjectRepository Instance
        {
            get
            {
                if (_instance == null)
                {
                    lock (_locker)
                    {
                        if (_instance == null)
                        {
                            _instance = new ProjectRepository();
                            _instance.component.StartListen();
                        }
                    }
                }
                return _instance;
            }
        }

        public void InitializeRepos()
        {
            //All vehicles
            #region

            lock (_locker)
            {
                projectPlanRepo = new DataSet();
                thePlans = projectPlanRepo.Tables.Add("Plans");
                theProjectPlans.Columns.Add("PlanGuid");
                theProjectPlans.Columns.Add("PlanID");
                theProjectPlans.Columns.Add("PlanName");
                theProjectPlans.Columns.Add("PlanStartTime", typeof(DateTime));

                //set the primary key to the vehicleID Column
                DataColumn[] keys = new DataColumn[1];
                keys[0] = theProjectPlans.Columns["PlanID"];
                theProjectPlans.PrimaryKey = keys;

                theTasks = projectPlanRepo.Tables.Add("Tasks");
                theTasks.Columns.Add("TaskGuid");
                theTasks.Columns.Add("PlanID");
                theTasks.Columns.Add("TaskName");
                theTasks.Columns.Add("TaskStartTime", typeof(DateTime));
                theTasks.Columns.Add("TaskDuration", typeof(TimeSpan));
                theTasks.Columns.Add("ParentTaskGuid");
                theTasks.Columns.Add("TaskPercentComplete");
                theTasks.Columns.Add("TaskType");    // Stores whether this is a mission, phase, or base task
                // Any other Task properties not covered above are stored as Binary data in the AllProperties field. 
                // This includes properties like Resources, Predecessors.
                theTasks.Columns.Add("AllProperties", typeof(Byte[]));

                //set the primary key to the tasksGuid Column
                DataColumn[] tasksKey = new DataColumn[1];
                tasksKey[0] = theTasks.Columns["TaskGuid"];
                theTasks.PrimaryKey = tasksKey;

                System.Data.DataRelation relOrdDet;
                System.Data.DataColumn colMaster;
                System.Data.DataColumn colDetail;
                colMaster = projectPlanRepo.Tables["Plans"].Columns["PlanID"];
                colDetail = projectPlanRepo.Tables["Tasks"].Columns["PlanID"];
                relOrdDet = new System.Data.DataRelation("RelOrdDet", colMaster, colDetail);
                projectPlanRepo.Relations.Add(relOrdDet);

                // Now create a default project workspace
                DataRow newRow = theProjectPlans.NewRow();
                Guid thisGuid = Guid.NewGuid();
                newRow["PlanGuid"] = thisGuid;
                newRow["PlanID"] = defaultProjectName;
                newRow["PlanName"] = defaultProjectName;
                newRow["PlanStartTime"] = DateTime.Now;
                theProjectPlans.Rows.Add(newRow);
            }
            #endregion
        }
    }
}
